@@ -42,6 +42,7 @@ void LottieLoader::run(unsigned tid)
         comp = parser.comp;
         builder->build(comp);
     }
+    rebuild = false;
 }
 
 
@@ -164,7 +165,7 @@ bool LottieLoader::header()
         ++p;
     }
 
-    if (frameRate < FLT_EPSILON) {
+    if (frameRate < FLOAT_EPSILON) {
         TVGLOG("LOTTIE", "Not a Lottie file? Frame rate is 0!");
         return false;
     }
@@ -294,14 +295,14 @@ bool LottieLoader::override(const char* slot)
 
         if (idx < 1) success = false;
         free(temp);
-        overriden = success;
-
+        rebuild = overriden = success;
     //reset slots
     } else if (overriden) {
         for (auto s = comp->slots.begin(); s < comp->slots.end(); ++s) {
             (*s)->reset();
         }
         overriden = false;
+        rebuild = true;
     }
     return success;
 }
@@ -361,6 +362,8 @@ float LottieLoader::duration()
 void LottieLoader::sync()
 {
     this->done();
+
+    if (rebuild) run(0);
 }
 
 
